import logging
from utils import Utils
from typing import Optional
from settings import load_env_var
import os 
from pathlib import Path
import joblib
import pandas as pd
import datetime
import hopsworks
class ErrorLoadingModel(Exception):
    def __init__(self,message="Error Loading Model"):
        self.message = message
        super().__init__(self.message)


class ErrorWritingToAws(Exception):
    def __init__(self,message="Error in aws"):
        self.message = message
        super().__init__(self.message)


class Batch():
    def __init__(self,
                logger:logging.Logger,
                 ):
            load_env_var("/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/batch-prediction-pipeline/batch-prediction-pipelines")
            self.logger  = logger
            self.api_key = os.environ.get("HOPS_WORKS_API")
            self.project_name = os.environ.get("HOPSWORKS_PROJECT_NAME")
            self.feature_store_name = os.environ.get("")

    
    def predict(
            self,
    fh: int = 24,
    feature_view_version: Optional[int] = None,
    model_version: Optional[int] = None,
    start_datetime: Optional[datetime] = None,
    end_datetime: Optional[datetime] = None,
) -> None:
            """Main function used to do batch predictions.
        
            Args:
                fh (int, optional): forecast horizon. Defaults to 24.
                feature_view_version (Optional[int], optional): feature store feature view version. If None is provided, it will try to load it from the cached feature_view_metadata.json file.
                model_version (Optional[int], optional): model version to load from the model registry. If None is provided, it will try to load it from the cached train_metadata.json file.
                start_datetime (Optional[datetime], optional): start datetime used for extracting features for predictions. If None is provided, it will try to load it from the cached feature_pipeline_metadata.json file.
                end_datetime (Optional[datetime], optional): end datetime used for extracting features for predictions. If None is provided, it will try to load it from the cached feature_pipeline_metadata.json file.
            """
        
            if feature_view_version is None:
                feature_view_metadata = Utils.load_json("feature_view_metadata.json")
                feature_view_version = feature_view_metadata["feature_view_version"]
            if model_version is None:
                train_metadata = Utils.load_json("train_metadata.json")
                model_version = train_metadata["model_version"]
            if start_datetime is None or end_datetime is None:
                feature_pipeline_metadata = Utils.load_json("feature_pipeline_metadata.json")
                start_datetime = datetime.strptime(
                    feature_pipeline_metadata["export_datetime_utc_start"],
                    feature_pipeline_metadata["datetime_format"],
                )
                end_datetime = datetime.strptime(
                    feature_pipeline_metadata["export_datetime_utc_end"],
                    feature_pipeline_metadata["datetime_format"],
                )
        
            self.logger.info("Connecting to the feature store...")
            project = hopsworks.login(
                api_key_value=os.environ.get("HOPS_WORKS_API"), project=os.environ.get("HOPSWORKS_FS_NAME")
            )
            fs = project.get_feature_store()
            self.logger.info("Successfully connected to the feature store.")
        
            self.logger.info("Loading data from feature store...")
            self.logger.info(f"Loading features from {start_datetime} to {end_datetime}.")
            X, y = data.load_data_from_feature_store(
                fs,
                feature_view_version,
                start_datetime=start_datetime,
                end_datetime=end_datetime,
            )
            self.logger.info("Successfully loaded data from feature store.")
        
            self.logger.info("Loading model from m odel registry...")
            model = self.load_model_from_model_registry(project, model_version)
            self.logger.info("Successfully loaded model from model registry.")
        
            self.logger.info("Making predictions...")
            predictions = forecast(model, X, fh=fh)
            predictions_start_datetime = predictions.index.get_level_values(
                level="datetime_utc"
            ).min()
            predictions_end_datetime = predictions.index.get_level_values(
                level="datetime_utc"
            ).max()
            self.logger.info(
                f"Forecasted energy consumption from {predictions_start_datetime} to {predictions_end_datetime}."
            )
            self.logger.info("Successfully made predictions.")
        
            self.logger.info("Saving predictions...")
            save(X, y, predictions)
            self.logger.info("Successfully saved predictions.")
        
            # Save the predictions to the bucket for monitoring.
            self.logger.info("Merging predictions with cached predictions...")
            self.save_data_for_monitoring(predictions, start_datetime)
            self.logger.info("Successfully merged predictions with cached predictions...")


    def load_model_from_registery(self,model_version:Optional[int]=1):
        try:
            #login to hopswork
            fs = Utils.login_to_hopsworks(
                logger=self.logger,
                api_key=self.api_key,
                project_name=self.project_name,
                feature_store_name=os.environ.get("HOPSWORKS_FS_NAME")
            )
            model_registry = fs.get_model_registry()
            model = model_registry.get_model(name="best_model",version=model_version)
            model_dir = model.download()
            model_path = Path(model_dir) / "best_model.pk1"
            m = joblib.load(model_path)
            return m
        except ErrorLoadingModel as e:
            self.self.logger.error(f"Couldn't Load The model from feature store please recheck {e}")

    def forecast_model_multivalues(self,model,X_values:pd.DataFrame,fh:Optional[int]=24):
        try:
            # we have 3 level indexes based on our data The first is area , consumers abd latest datetime
            all_areas = X_values.index.get_level_values(level=0).unique()
            all_consumer =X_values.index.get_level_values(level=1).unique()
            latest_datetime = X_values.index.get_level_values(level=2).max()
            start = latest_datetime+1
            end = start+fh-1
            fh_range = pd.date_range(
                start=start,end=end,freq="H"
            )
            index = pd.MultiIndex.from_product(
                [all_areas,all_consumer,fh_range],names=["areas","consumers_type","datetime_utc"]
            )
            X_forecast = pd.DataFrame(index=index)
            X_forecast["area_exog"] = X_forecast.index.get_level_values(0)
            X_forecast["consumer_type_exog"] = X_forecast.index.get_level_values(1)
            predictions = model.predict(X_forecast)
            return predictions
        except Exception as e:
            self.self.logger.error("Error while working on The data and doing The multiindexing")
            return None 
        
    def save(self,X:pd.DataFrame,y:pd.DataFrame,predictions:pd.DataFrame):
        """ save The Input data and target data  into aws """
        try:
            self.self.logger.info("Writting The data into aws")
            for df, blob_name in zip(
                [X,y,predictions],["X.parquet", "y.parquet", "predictions.parquet"]
            ):
                self.self.logger.info(f"saving {blob_name} into aws") 
                Utils.write_to_aws(self.self.logger,
                                   os.environ.get("AWS_BUCKET_NAME"),
                                   data=df,
                                   key = blob_name)
                self.self.logger.info("successfuly saved")
        except ErrorWritingToAws as error:
            self.self.logger.error(f"couldn't write into aws {error}")
    
    def save_data_for_monitoring(self,preidction:pd.DataFrame,start_date):
        try:
            result = Utils.read_from_aws(self.self.logger,os.environ.get("AWS_BUCKET_NAME"),key="predictions.parquet")
            has_cached_prediction = result is not None
            if has_cached_prediction is True: 
                result.index = result.index.set_index(
                    pd.to_datetime(result.index.levels[2],unit="h").to_period("H"),
                    level=2
                )
                megerged_prediction = preidction.merge(
                    result,
                    left_index=True,
                    right_index=True,
                    how="outer",
                    suffixes=("_new","_cached")
                )
                new_predictions  = megerged_prediction.filter(regex=".*?_new")
                new_predictions.columns = new_predictions.columns.str.replace("_new","")
                result = megerged_prediction.filter(regex=".*?_cached")
                result.columns = result.columns.str.replace("_cached","")
                new_predictions.update(result)
                preidction = new_predictions
            preidction = preidction.loc[
                preidction.index.get_level_values("datetime_utc")>=pd.Period(start_date,frep="H")
            ]
            preidction.dropna(subset=["energy_consumption"],inplace=True)
            Utils.write_to_aws(
                self.self.logger,
                os.environ.get("AWS_BUCKET_NAME"),
                data=preidction,
                key="new_pred"
            )
            self.self.logger('SUCCESSFULLY WROTE EVERYTHING TO AWS!!')
        except Exception  as e:
            self.self.logger.error("Can't type into zzz")


        
    



    

