import pymongo
import pandas as pd
import numpy as np 
import json
import logging
import os 
from settings import load_env_var
from datetime import timedelta
def connect_to_mongodb(db_name,collection_name):
    try:
        client = pymongo.MongoClient("mongodb://localhost:27017/")
        mydb = client[db_name]
        collection = mydb[collection_name]
        return collection
    except pymongo.errors.ConnectionFailure as e:
        # logs.error("Couldn't connect to mongodb please review your connection strings")
        return None
    
def Bulk_insert_into_mongodb(logs:logging.Logger,data:pd.DataFrame,collection):
    try:
        collection.insert_many(data.to_dict(orient="records"))
    except pymongo.errors.WriteError as e:
        print(e)
        logs.error("Couldn't write succesfully into mongodb please check your data format and be sure its a dict or json file")
        
def load_data_batch_mongo(logs:logging.Logger,collection:pymongo.MongoClient,datetime):
    try:
        start_dt = pd.to_datetime(datetime).replace(hour=0,minute=0,second=0,microsecond=0)
        end_dt = start_dt + timedelta(days=1)
        result = collection.find({"datetime_utc":{
            "$gte":start_dt,
            "$lt":end_dt
        }})
        yield result         
    except pymongo.errors.OperationFailure as e:
        logs.error(f"Operatoion of reading as batches lead to a failure please check your connection params: {e}")
        print(f"Operatoion of reading as batches lead to a failure please check your connection params: {e}")

def filter_date_mongo_data(logs:logging.Logger,collection:pymongo.MongoClient):
    try:
        filter =  collection.find({},{"datetime_utc":1,"_id":0})
        dates = [doc.get("datetime_utc").date() for doc in filter]
        dates = sorted(dates)
        return pd.Series(dates).drop_duplicates().to_numpy()
    except pymongo.errors.OperationFailure as e:
        logs.error(f"Couldn't filter The data please make sure all The required params are statisfied : {e}")  
        return None    

# if __name__ == "__main__":
#     load_env_var("/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/batch-prediction-pipeline/")
#     collection = connect_to_mongodb(os.environ.get('MONGODB_DB_NAME'),os.environ.get("MONGO_DB_COLLECTION_NAME"))
#     a = filter_date_mongo_data(collection)
#     print(a)