import logging 
import json
import os
import settings
from pathlib import Path
from typing import Optional
import pandas as pd
import boto3
from typing import Union
import uuid
from settings import load_env_var
import hopsworks 
class ErrorLoginToFeatureStore(Exception):
     def __init__(self,message):
          self.message = message
          super().__init__(self.message)



class Utils():
    
    @staticmethod
    def get_loggers(
            name:str    
        ) -> logging.Logger:
            """Function to save Logs for future analysis and use"""
            try:
                logger = logging.getLogger(name)
                logging.basicConfig(filename="../"+os.getcwd()+"/outputs/logs.log",encoding="utf-8",level=logging.INFO)
                return logger
            except Exception as e:
                 #create the folder
                 pass
    
    @staticmethod
    def save_json(
             data: dict , file_name : str , file_dir : str=settings.OUTPUT_DIR
        ):
            data_path = Path(file_dir) / file_name
            try:
                with open(data_path,"w") as f:
                    json.dump(data,f)
            except   Exception as e:
                raise "couldn't save The data into our system !"   

    @staticmethod
    def load_json(
                file_name : str , file_dir : str  = settings.OUTPUT_DIR
        ):
            data_path = Path(file_dir)/file_name
            if not data_path.exists():
                raise FileNotFoundError(f"Cached JSON frame doesn't exists")
            with open(data_path,"r") as f :
                data = json.load(f)
            return data

   

    @staticmethod
    def write_to_aws(logs:logging.Logger,bucket_name:Optional[str],data:Union[json,pd.DataFrame],key:str):
         try:
              #we need to return The whole bucket , re adjust it and just send it back to s3 (expensive)
              s3_client = boto3.client("s3")
          #     rd_uuid = uuid.uuid1()
              z = s3_client.put_object(
                   Bucket = bucket_name,
                   Key = f"predictions/{key}",
                   Body = json.dumps(data),
                   ContentType = "application/json"
              )  
              logs.info("Writing data into s3 done succesfully")
         except Exception as error:
              logs.error(f"Writing data into s3 went wrong : {error}")
              
              
    @staticmethod
    def read_from_aws(logs:logging.Logger,bucket_name:Optional[str],key:Optional[str]="predictions.parquet"):
         try:
              s3_client = boto3.client("s3")
              logs.info("Reading Data from s3 ...")
              if key is not None:
                    result = s3_client.get_object(
                         Bucket =  bucket_name,
                         Key = f"predictions/{key}"
                    )
                    logs.info("succesfully read The data from s3!!")
              else:
                   logs.info("couldn't read from aws succesfully ")
                   return None
              
              return result
             
         except Exception as e:
              logs.error(f"Couldn't read The Bucket Please make sure you enetered The right key or bucket: {e}")
                    
    @staticmethod
    def login_to_hopsworks(logger:logging.Logger,
                           api_key:Optional[str],
                           project_name:Optional[str],
                           feature_store_name:Optional[str]):
         try:
              project = hopsworks.login(project=project_name,
                                       api_key_value=api_key
                                       )
              fs = project.get_feature_store(name=feature_store_name)
              return fs
         except ErrorLoginToFeatureStore as error:
              logger.error(f"Couldn't Login to feature Store please recheck {error}")
              
         
# if __name__ == "__main__":
#     logger = logging.getLogger(__name__)
#     logging.basicConfig(filename=os.getcwd()+"/outputs/logs.log",encoding="utf-8",level=logging.INFO)
#     u = Utils.write_to_aws(logger,"energyforecaster12",{"name":"baker"})
#     print(u)
#     z = Utils.read_from_aws(logger,"energyforecaster12",str(u))


    

   