from json import JSONDecodeError
import pandas as pd 
import os 
import sys 
from pathlib import Path 
from typing import Tuple,Optional,Dict,Any
import requests
from datetime import datetime ,  timedelta
import logging
sys.path.append(str(Path(__file__).parent.parent))
import json
from utils import Utils
from settings import SETTINGS
class Extract:
        
        def __init__(self,logger:logging.Logger,file_dir_metdata:Optional[str],file_dir_data:Optional[str]="/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/feature_pipeline/feature_pipeline/outputs/data/Consumption_energy.json",url:str = "https://api.energidataservice.dk/dataset/ConsumptionDE35Hour"):
               self.logger = logger
               self.url = url
               self.file_dir_metadata = file_dir_metdata
               self.dir_data = file_dir_data

        def from_api(
                self,
                export_end_reference:datetime,
                days_delay:int=10,
                days_export:int=15,
                dateformat:str = "%Y-%m-%d %H:%M"   
                )->Optional[Tuple[pd.DataFrame,Dict[str,Any]]]:
            export_start , export_end = self._compute_extraction_window(export_end_reference,days_delay,days_export)
            
            try:

                response = requests.get(self.url)
                status = response.status_code
                if status!=200:
                    self.logger.error(f"Response Status = {status}. Could not download the file.")
                    return None 
                data = pd.DataFrame(response.json()["records"])
                metadata = {
                "days_delay": days_delay,
                "days_export": days_export,
                "url": self.url,
                "export_datetime_utc_start": export_start.strftime(dateformat),
                "export_datetime_utc_end": export_end.strftime(dateformat),
                "datetime_format": dateformat,
                "num_unique_samples_per_time_series": len(data["HourUTC"].unique()),
                "date_of_operation":datetime.now().isoformat()
            }
                # data.to_json(self.dir_data)
                # Utils.save_json(metadata,file_name="metdata.json",file_dir=self.file_dir_metadata)
                self.logger.info("saved The data into cache metadata")
                return data,metadata
            except JSONDecodeError as e:
                self.logger.error(f"couldn't parse The response as JSON. {e}")
                return None , None 
            except Exception as e:
                self.logger.error(f"An error occured while downloading The data from The API. {e}")
                return None , None

        def from_file(
                self,
                export_end_reference_datetime:datetime=None,
                days_delay:int = 10,
                days_export:int = 15,
                cached_dir:Optional[Path]=None,
                dateformat:str="%Y-%m-%d %H:%M",
        ) -> Optional[Tuple[pd.DataFrame,Dict[str,Any]]]:
            #Bring The data between The 2 dates 
            export_start ,  export_end = self._compute_extraction_window(export_end_reference_datetime , days_delay,days_export)
            records = self._extract_data_from_file_url(self.url,export_start,dateformat,export_end,cached_dir)
            metadata = {
                "days_delay": days_delay,
                "days_export": days_export,
                "url": self.url,
                "export_datetime_utc_start": export_start.strftime(dateformat),
                "export_datetime_utc_end": export_end.strftime(dateformat),
                "datetime_format": dateformat,
                "num_unique_samples_per_time_series": len(records["HourUTC"].unique()),
                "feature_group_version":1,
                "date_operation_done":datetime.now()
            }
            self.logger.info("saved The data into cache metadata")
            print("done")
            return records, metadata
        
        
        def _extract_data_from_file_url(self,
                                        url:str ,
                                        export_start:datetime,
                                        datetime_format:str,
                                        export_end:datetime,
                                        cached_dir:Optional[Path]) -> Optional[pd.DataFrame]:
            if cached_dir is None:
                cached_dir = Path("/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/feature_pipeline/feature_pipeline/outputs/data")
                cached_dir.mkdir(parents=True,exist_ok=True)
                
            filePath  = cached_dir/ "ConsumptionDE35Hour.json"
            if not filePath.exists():
                self.logger.info("...Downloading The data from Url...")
                try:
                    response = requests.get(url)
                    print(response)
                except requests.exceptions.HTTPError as e:
                    print("none")
                    self.logger.error(
                        f"Trying To Download The data but {e}"
                    )
                    return None
                if response.status_code!=200:
                    self.logger.error(f"Response status = {response.status_code}. Could not download the file.")
                    return None
                with filePath.open("w") as file:
                    json.dump(response.json()["records"],file)
                self.logger.info("Succesfully downloaded The data into The file")
            else:
                self.logger.info("Data already exists in The file path")
                try:
                    data = pd.read_json(filePath)
                    return data
                except pd.errors.EmptyDataError as e:
                    filePath.unlink(missing_ok=True)
                    self.logger.error(f"Downloaded file at {filePath} is empty. Could not load it into a DataFrame.")
                    return None
                # records  = data[(data["HourUTC"] >= export_start.strftime(datetime_format)) & (data["HourUTC"] <= export_end.strftime(datetime_format))]
                
        
        def _compute_extraction_window(self,export_end_reference_datetime: datetime , days_delay: int , days_export: int) -> Tuple[datetime,datetime]:
            if export_end_reference_datetime is None:
                export_end_reference_datetime = datetime(2023,6,30,21,0,0) + timedelta(days=days_delay)
            export_end_reference_datetime = export_end_reference_datetime.replace(minute=0,second=0,microsecond=0)
            expiring_dataset_datetime = datetime(2023,6,30,21,0,0) + timedelta(days=days_delay)
            if export_end_reference_datetime>expiring_dataset_datetime:
                export_end_reference_datetime = expiring_dataset_datetime
                self.logger.warning( "We clapped 'export_end_reference_datetime' to 'datetime(2023, 6, 30) + datetime.timedelta(days=days_delay)' as \
                the dataset will not be updated starting from July 2023. The dataset will expire during 2023. \
                Check out the following link for more information: https://www.energidataservice.dk/tso-electricity/ConsumptionDE35Hour")
        
            export_end = export_end_reference_datetime - timedelta(days_delay)
            export_start = export_end_reference_datetime - timedelta(days_delay+days_export)
            min_export  = datetime(2023,6,30,22,0,0)
            if export_start  < min_export : 
                export_start = min_export 
                export_end = export_start  + timedelta(days_export)
                self.logger.warning(
                    "We clapped 'export_start' to 'datetime(2020, 6, 30, 22, 0, 0)' and 'export_end' to 'export_start + datetime.timedelta(days=days_export)' as this is the latest window available in the dataset."
                )
            return export_start , export_end
            
# if __name__ == "__main__":
#     logger = Utils.get_loggers(__name__)
#     extract = Extract(logger,file_dir="/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/feature_pipeline/feature_pipeline/outputs/data")
#     extract.from_api(datetime(2023,6,30,21,0,0),10,15)