import hopsworks
import pandas as pd
import logging
import os
from typing import Optional
import sys
from pathlib import Path
sys.path.append(str(Path(__file__).parent.parent))
from datetime import datetime
from settings import load_env_var
class Loads:
    def __init__(self,
                 logger: logging.Logger,
                 version: int,
                 description: Optional[str],
                 project_name: Optional[str],
                 feature_gp_name: str="energy_forecast"):
        
        load_env_var("../")
        self.logger = logger
        self.api_key = os.environ.get("HOPS_WORKS_API")
        self.feature_gp_name = feature_gp_name
        self.version = version
        self.description = description
        self.project_name = project_name

    def _connecting_to_hopsworks(self):
        try:
            project = hopsworks.login(
                port=443,
                project=self.project_name,
                api_key_value=self.api_key
            )
            return project
        except Exception as e:
            self.logger.exception("Error connecting to Hopsworks")
            return None

    def inserting_to_feature_store(self, data: pd.DataFrame):
        try:
            project = self._connecting_to_hopsworks()
            print("heyyy bbb")
            if project is None:
                return 

            fs = project.get_feature_store()
            try:
                fg = fs.get_feature_group(name=self.feature_gp_name, version=self.version)
                
            except:
                
                fg = fs.create_feature_group(
                    name=self.feature_gp_name,
                    version=self.version,
                    description=self.description,
                    primary_key=["energy_consumption"],
                    online_enabled=True,
                    event_time='datetime_utc'
                )
            self.logger.info("Inserting began to Hopsworks")
            print("Inserting began to Hopsworks")
            fg.insert(data,write_options={"storage": "offline_and_online"}) 
            
            #save data into feature Store
            feature_view = fs.get_feature_view(name="energy_consumption_denmark_view_new",version=1)
            
            
            # Then call create_training_data properly
            feature_view.create_training_data(
                data_format="csv",  # or "tfrecord", etc.
                description="Training data for energy forecasting",
                # data=data
            )
            self.logger.info("Successfully inserted in Hopsworks")
        except Exception as e:
            self.logger.exception("Error inserting data to feature store")

    def loading_from_feature_store(self) -> Optional[list]:
        max_retries = 2
        for attempt in range(max_retries):
            try:
                project = self._connecting_to_hopsworks()
                if project is None:
                    continue

                fs = project.get_feature_store()
                fg = fs.get_feature_group(name=self.feature_gp_name, version=self.version)
                return fg, fs
            except Exception as e:
                self.logger.error(f"Attempt {attempt + 1}/{max_retries} failed: {str(e)}")
                if attempt == max_retries - 1:
                    self.logger.error("All attempts to load from feature store failed")
                    return None
                self.logger.info("Retrying connection to feature store...")


# if __name__ == "__main__":
#     load_env_var("/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/feature_pipeline/.env")
#     #Extract The data and insdert to feature Store
#     logger = Utils.get_loggers(__name__)
#     extract = Extract(logger,file_dir_metdata="/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/feature_pipeline/feature_pipeline/outputs/data")
#     result,metadata = extract.from_api(datetime(2023,6,30,21,0,0),10,15)
    

    
