from typing import Optional
from datetime import datetime
import utils
import logging
import os
import hopsworks
from settings import load_env_var
from etl.loads import Loads
import pandas as pd
# from hopsworks.exceptions import FeaturestoreException

class FeatureView:
    load_env_var("../")
    def __init__(self,logger:logging.log,data:pd.DataFrame):
             self.logs = logger
             self.loading_json = utils.Utils().load_json("feature_pipeline_metadata.json","outputs")
             self.api_key_value = os.environ.get("HOPS_WORKS_API")
             self.data = data
             self.load = Loads(
                  self.logs,
                  feature_gp_name="energy_forecast",
                  version=1,
                  data = self.data,
                  description="Energy consumption forecasting feature group",
                  project_name="driver_score",
                  )

    def create(
        self,
        start_extraction_window :Optional[datetime]= None,
        end_extraction_window:Optional[datetime]= None,
        project_name:Optional[str]= "driver_score",
        feature_gp_name : Optional[str]= "energy_forecast",
        feature_gp_version:Optional[int]= 1,
        feature_store_name : Optional[str] = "driver_score_featurestore"
    )->dict:
        try: 
            if feature_gp_version is None:
                feature_gp_version  = self.loading_json["feature_group_version"]
    
    
            if start_extraction_window is None:
                 start_extraction_window = datetime.strptime(
                      self.loading_json["export_datetime_utc_start"],
                      self.loading_json["datetime_format"]
                    )
    
    
            if end_extraction_window is None:
                 end_extraction_window = datetime.strptime(
                      self.loading_json["export_datetime_utc_end"],
                      self.loading_json["datetime_format"]
                    )

            fg ,fs  = self.load.loading_from_feature_store()
            ds_query = fg.select_all()
            feature_view = fs.create_feature_view(
                 name="energy_consumption_denmark_view",
                 description="Energy consumption for Denmark forecasting model.",
                 query=ds_query,
                 labels=[],
            )
            feature_df , label_df = feature_view.create_training_data(
                 description ="Energy consumption for Denmark forecasting model.",
                 data_format = 'csv',
                 write_options = {"wait_for_job": False},
                 start_time = start_extraction_window,
                 end_time = end_extraction_window,
                 coalesce=False,
            )
            metadata = {
                 "feature_view_version": feature_view.version,
                 "training_dataset_version": 1,
            }
            #TODO -- save The METADAT AS JSON USING tHE SAVE JSON IN utils. insert to database  
            return metadata
            
        
        except Exception as e:
             self.logs.error(f"Sorry we had an error in The Feature Store or Feature Group {e}")
             return None 
        

"""Tested Working succesfuly"""      
# if __name__ == "__main__":
#     logger = logging.getLogger(__name__)
#     logging.basicConfig(filename=os.getcwd()+"../outputs/logs.log",encoding="utf-8",level=logging.INFO)
#     fs = FeatureView(logger)
#     meatdata = fs.create()
#     print(meatdata)

        



        
