import pandas as pd
import numpy as np
from datetime import datetime, timedelta
CONSUMER_TYPES  = [111, 112, 119, 121, 122, 123, 130, 211, 212, 215, 220, 310, 320, 330, 340, 350, 360, 370, 381, 382, 390, 410, 421, 422, 431, 432, 433, 441, 442, 443, 444, 445, 446, 447, 450, 461, 462, 999]
#we are going to use this one instead of The real data 
class Generate_Data:
    @staticmethod
    def generate_synthetic_time_series(
        start_date='2023-06-01',
        end_date='2023-06-30',
        freq='H',  # Hourly
        area=1,
        consumer_types=CONSUMER_TYPES,
        base_consumption=50000,
        trend=0.01,
        seasonality_strength=0.3,
        noise_level=0.1
    ):
        """
        Generate synthetic time series data that mimics your structure
        """
        
        # Create datetime range
        date_range = pd.date_range(
            start=start_date, 
            end=end_date, 
            freq=freq,
            tz='UTC'
        )
        
        # Generate data for each consumer type
        synthetic_data = []
        
        for consumer_type in consumer_types:
            # Create base pattern with trend and seasonality
            n_points = len(date_range)
            
            # Base consumption level (different for each consumer)
            base_level = base_consumption * (0.5 + np.random.random() * 1.5)
            
            # Add trend
            trend_component = np.linspace(0, trend * n_points, n_points) * base_level
            
            # Add daily seasonality (higher during day, lower at night)
            hours = np.array([dt.hour for dt in date_range])
            daily_pattern = np.sin(2 * np.pi * (hours - 6) / 24) * seasonality_strength * base_level
            
            # Add weekly seasonality (higher on weekdays)
            days_of_week = np.array([dt.weekday() for dt in date_range])
            weekly_pattern = (5 - days_of_week) / 5 * 0.1 * base_level
            
            # Add noise
            noise = np.random.normal(0, noise_level * base_level, n_points)
            
            # Combine all components
            consumption = base_level + trend_component + daily_pattern + weekly_pattern + noise
            
            # Ensure no negative values
            consumption = np.maximum(consumption, base_level * 0.1)
            
            # Create records for this consumer type
            for i, (timestamp, value) in enumerate(zip(date_range, consumption)):
                synthetic_data.append({
                    'area': area,
                    'consumer_type': consumer_type,
                    'datetime_utc': timestamp,
                    'energy_consumption': int(value)
                })
        
        # Create DataFrame
        df = pd.DataFrame(synthetic_data)
        metadata = {
                "days_delay": 10,
                "days_export": 15,
                "url": "Generated_data",
                "export_datetime_utc_start": start_date,
                "export_datetime_utc_end": end_date,
                "num_unique_samples_per_time_series": len(df),
                "feature_group_version":1,
                "date_operation_done":datetime.now()
            }
        
        # Set MultiIndex to match your structure
        # df = df.set_index(['area', 'consumer_type', 'datetime_utc'])
        
        return df,metadata