import logging
from typing import Optional
import wandb
import os 
from utils import Utils
from pathlib import Path
import json
class WandLoginError(Exception):
    pass
    

class BestConfig():
    def __init__(self,
                 logger:logging.Logger,
                  name:Optional[str],
                  project_name :Optional[str]=None,
                  group:Optional[str]=None,
                  job_type:Optional[str]=None,
                  entity:Optional[str]=None,
                  resume:Optional[str]="allow",
                  reinit:Optional[bool]=False,
                  run_id:Optional[int]=None,
                  add_timestamp_to_name:Optional[bool]=False
                  ):
        self.logger = logger
        self.api_key = os.environ.get
        self.name = name
        self.project_name = project_name
        self.group = group
        self.job_type = job_type
        self.entity = entity
        self.resume = resume
        self.reinit = reinit
        self.run_id = run_id
        self.add_timestamp_to_name = add_timestamp_to_name
    def upload(self,sweep_id:Optional[str]):
        try:

            self.logger("Starting loading The best sweep to wandb artifact")
            try:
               wandb.login()
               api = wandb.Api
            except WandLoginError as e:
                self.logger.error(WandLoginError("Error while login to wandb"))
            #getting Thebest sweep 
            sweep = api.sweep(f"risers/energy_forecasting/sweeps/{sweep_id}")
            best_run = sweep.best_run()
            # writing The sweep to Artifacts
            with Utils.init_wandb(
               entity=self.entity,
               group=self.group,
               job_type=self.job_type,
               resume=self.resume,
               reinit=self.reinit,
               id=self.run_id,
               project = self.project_name,

            ) as run:
                run.use_artifact("config:latest")
                best_rconfig = dict(run.config)
                self.logger.info(f"Best run is {best_run.name}")
                self.logger.info("Best config is: ")
                self.logger.info(best_rconfig)
            #send to sql 
                config_path = Path("/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/training_pipeline/outputs")
                if config_path is None:
                    os.mkdir("/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/training_pipeline/outputs")
                cached_file = config_path + "config_sweep.json"
                with open(cached_file,"wr") as file:
                    json.dump(best_rconfig,file,indent=4)
                artifact  = wandb.Artifact(
                    name = "best_config",
                    type="model",
                    metadata = {"results":{"validation":dict(run.summary["validation"])}},
                )
                artifact.add_file(str(cached_file))
                run.log_artifact(artifact)
                run.finish()
        except Exception as e:
            self.logger.error("Error Occured while doing The artifacts")
            return None
