import logging 
from typing import Optional
from data import Data
from exceptions import DataRetrieveingError
import pandas as pd
from configs import sweep_configs
import wandb
import os 
from utils import Utils
from functools import partial
from models import Models
import matplotlib.pyplot as plt
from sktime.forecasting.model_selection import ExpandingWindowSplitter
from sktime.forecasting.model_evaluation import evaluate as cv_evaluate
from sktime.performance_metrics.forecasting import MeanAbsolutePercentageError
from sktime.utils.plotting import plot_windows
from sktime.forecasting.model_selection import temporal_train_test_split
from settings import load_env_var
from settings import OUTPUT_DIR
from sktime.datatypes import check_raise
import numpy as np
import matplotlib
from generating_data import Generate_Data
class HyperParamTuning():
     
     def __init__(self,logger:logging.Logger,version:int,description:Optional[str],project_name:str,fh:int,entity:Optional[str]):
        
        self.logger = logger
        self.version = version
        self.description = description
        self.project_name = project_name
        self.fh = fh
        self.entity = entity

     def run(self,
            name:Optional[str]=None,
            group:Optional[str]=None,
            job_type:Optional[str]=None,
            run_id:Optional[int]=None,
            )->None:
        #loading The data from feature store with the required Version 
        try:
            try:
                    self.data = Data(self.logger,self.version,self.description,self.project_name)
                    y_train,_,X_train,_= self.data.load_data_from_feature_store(
                           name,
                           self.project_name,
                           group,
                           job_type,
                           self.entity,
                           run_id=run_id
                           )    
                    print(y_train)            
                    sweep_id = self.hyperparams_optimization(y_train,X_train)
                    metadata = {"sweep_id":sweep_id}
                    Utils.save_json(metadata,"last_sweep_metdata.json")
                    return metadata
            except DataRetrieveingError as e:
                self.logger.error("Couldn't retrive The data from feature store")
                return None 
            
        except Exception as e:
             print("sthg wring")
             return None
        
     def hyperparams_optimization(
                self,  y_train:pd.DataFrame,X_train:pd.DataFrame
        ):
            try:
                 #creating a new sweep using w&b
                 #login into wandb first 
                 wandb.login(key=os.environ.get("WDB_API_KEY"))
                 sweep_id = wandb.sweep(sweep_configs,project = os.environ.get("WDB_PROJECT_NAME"))
                 
                 wandb.agent(sweep_id,function = partial(self.run_sweep,y_train,X_train),project = os.environ.get("WDB_PROJECT_NAME"),count=1)
                 print("Done")
                 return sweep_id
            except Exception as e:
                 self.logger.error(f"Couldn't run The sweep please check the sweep part {e}")
                 return None 
            
     def run_sweep(self,y_train,X_train):
             """
             run a single train and log everythin into wandb
             """
             with wandb.init(
                  entity=os.environ.get("WDB_ENTITY_NAME"),
                  project=os.environ.get("WDB_PROJECT_NAME"),
                  name="test"
             ) as run:
                  run.use_artifact("energy_consumption_denmark_feature_view_new:latest")
                  #return The config files and train The model 
                  config = dict(wandb.config) if wandb.config else {}
                  
                  model = Models.build_model(config)
                  model,results = self.train_model_cv(model,y_train,X_train)
                  wandb.log(results)
                  metadata = {
                       "experiment":{"name":run.name},
                       "config":config,
                  }
                  artifacts = wandb.Artifact(
                       name = f"config",
                       type = "model",
                       metadata = metadata
                  )
                  run.log(artifacts)
                  run.finish()
     def train_model_cv(self,
                        model,
                        y_train:pd.DataFrame,
                        X_train:pd.DataFrame,
                        fh:Optional[int]=3,
                        k:int=3):
             """"
             Function to perform training in a cross validation way 
             """
             print(y_train)
             data_length  = len(y_train.index.get_level_values(-1).unique())
            
             print(y_train)
          #    y_train = y_train.reset_index().drop(["area","consumer_type"],axis=1)
          #    y_train.index = y_train["datetime_utc"]
          #    print(type(y_train["datetime_utc"]))
          #    y_train.drop("datetime_utc",inplace=True,axis=1)     
          #    print(y_train)
             assert data_length >= 25 , "not enough data to perform The data splitting"
             cv_length = data_length//k#25
             initial_window = 10
             splitter = ExpandingWindowSplitter(fh=[1,2],initial_window=10,step_length=1)
             self.render_cv_scheme(splitter,y_train)
             print("proceeding here")
             results= cv_evaluate(y=y_train,
                                  cv=splitter,
                                  scoring=MeanAbsolutePercentageError(),
                                  return_data=True,
                                  forecaster = model)
             print("succeeed!!!!!")
             print(results)
             results = results.rename(
             columns={
                 "test_MeanAbsolutePercentageError": "MAPE",
                 "fit_time": "fit_time",
                 "pred_time": "prediction_time",
             }
             )
             mean_results = results[["MAPE", "fit_time", "prediction_time"]].mean(axis=0)
             mean_results = mean_results.to_dict()
             results = {"validation": mean_results}
         
             self.logger.info(f"Validation MAPE: {results['validation']['MAPE']:.2f}")
             self.logger.info(f"Mean fit time: {results['validation']['fit_time']:.2f} s")
             self.logger.info(f"Mean predict time: {results['validation']['prediction_time']:.2f} s")
         
             return model, results  
     

     
     

            
     
     # def render_cv_scheme(self, cv, y_train: pd.DataFrame) -> str:
     #      """Render the CV scheme used for training and log it to W&B."""
      
     #      for group_key, group_data in y_train.groupby(level=[0, 1]):
     #          if len(group_data) >= (cv.initial_window + max(cv.fh)):
     #              random_time_series = group_data.reset_index(level=[0, 1], drop=True)
     #              break
     #      else:
     #          print("No time series long enough to render CV scheme.")
     #          return ""
     #      print("done")
     #      plot_windows(cv, random_time_series)
      
     #      save_path = str(OUTPUT_DIR / "cv_scheme.png")
     #      plt.savefig(save_path)
     #      wandb.log({"cv_scheme": wandb.Image(save_path)})
      
     #      return save_path

     def render_cv_scheme(self,cv, y_train: pd.DataFrame) -> str:
              """Render the CV scheme used for training and log it to W&B."""
              random_time_series = (
                  y_train.groupby(by=["area","consumer_type"])
                  .get_group((1,111))
              )
              print(random_time_series)
              plot_windows(cv, y_train.reset_index()["energy_consumption"])
              plot_windows(cv, random_time_series)
              save_path = str(OUTPUT_DIR / "cv_scheme.png")
              plt.savefig(save_path)
              wandb.log({"cv_scheme": wandb.Image(save_path)})
              print("Done!")
              return save_path

        

if __name__ == "__main__":
     matplotlib.use("Agg")
     logger = logging.getLogger(__name__)
     load_env_var("/Users/bakr/Desktop/personal_projects/energy_forecasting/mlforecasting/training_pipeline/")
     x = HyperParamTuning(logger=logger,
                          version=1,
                          project_name=os.environ.get("HOPSWORKS_PROJECT_NAME"),
                          description="",
                          fh=24,
                          entity=os.environ.get("WDB_ENTITY_NAME"),
                          )
     w = x.run(name="test",group="datasets",job_type="prepare_dataset",run_id=3)
     print(w)