from sktime.forecasting.compose import make_reduction, ForecastingPipeline
from sktime.forecasting.naive import NaiveForecaster
from sktime.transformations.series.date import DateTimeFeatures
from sktime.transformations.series.summarize import WindowSummarizer
import lightgbm as lgb
from transformers import AttachAreaConsumerType

class Models:
    def __init__(self):
        pass

    @staticmethod
    def build_model(config: dict):
        # Extract window summarizer params from config
        lag = config.pop("forecaster_transformers__window_summarizer__lag_feature__lag", [[1, 24]])
        mean = config.pop("forecaster_transformers__window_summarizer__lag_feature__mean", [[1, 48]])
        std = config.pop("forecaster_transformers__window_summarizer__lag_feature__std", [[1, 72]])
        n_jobs = config.pop("forecaster_transformers__window_summarizer__n_jobs", 1)

        window_summarizer = WindowSummarizer(
            
            lag_feature={
                "lag": lag,
                "mean": mean,
                "std": std,
            },
            n_jobs=n_jobs,
        )

        regressor = lgb.LGBMRegressor()

        forecaster = make_reduction(
            regressor,
            transformers=[window_summarizer],
            strategy="recursive",
            pooling="global",
            window_length=None,
        )

        pipe = ForecastingPipeline(
            steps=[
                ("attach_area_and_consumer_type", AttachAreaConsumerType()),
                (
                    "daily_season",
                    DateTimeFeatures(
                        manual_selection=["day_of_week", "hour_of_day"],
                        keep_original_columns=True,
                    ),
                ),
                ("forecaster", forecaster),
            ]
        )

        # Set any additional model parameters
        try:
            pipe.set_params(**config)
        except ValueError as e:
            print(f"⚠️ Warning: {e}")
        
        return pipe

    @staticmethod
    def build_base_model(seasonal_periodictivity: int):
        return NaiveForecaster(sp=seasonal_periodictivity)
