import logging 
import json
import os
import settings
from pathlib import Path
import joblib
import wandb
from typing import Optional
import pandas as pd
from contextlib import contextmanager

class Utils():
    
    @staticmethod
    def get_loggers(
            name:str    
        ) -> logging.Logger:
            """Function to save Logs for future analysis and use"""
            try:
                logger = logging.getLogger(name)
                logging.basicConfig(filename="../"+os.getcwd()+"/outputs/logs.log",encoding="utf-8",level=logging.INFO)
                return logger
            except Exception as e:
                 #create the folder
                 pass
    
    @staticmethod
    def save_json(
             data: dict , file_name : str , file_dir : str=settings.OUTPUT_DIR
        ):
            data_path = Path(file_dir) / file_name
            try:
                with open(data_path,"w") as f:
                    json.dump(data,f)
            except   Exception as e:
                raise "couldn't save The data into our system !"   

    @staticmethod
    def load_json(
                file_name : str , file_dir : str  = settings.OUTPUT_DIR
        ):
            data_path = Path(file_dir)/file_name
            if not data_path.exists():
                raise FileNotFoundError(f"Cached JSON frame doesn't exists")
            with open(data_path,"r") as f :
                data = json.load(f)
            return data

    @staticmethod
    def save_model(model,file_name:Path):
        """
        function made for  saving The model into w&b
        """
        try:
             joblib.dump(model,file_name)
        except Exception as e:
             logs  = Utils.get_loggers(__name__)
             logs.error(f"couldn't save The model {e}")
    
         
    




    @staticmethod
    def load_model(file_name:Path):
        """
        function for loading The model for feature use 
        """
        model = joblib.load(file_name)
        return model
    @staticmethod
    @contextmanager
    def init_wandb(
              api_key:str,
              name:Optional[str],
              project_name :Optional[str]=None,
              group:Optional[str]=None,
              job_type:Optional[str]=None,
              entity:Optional[str]=None,
              resume:Optional[str]="allow",
              reinit:Optional[bool]=False,
              run_id:Optional[int]=None,
              add_timestamp_to_name:Optional[bool]=False,
              
    ):
         if add_timestamp_to_name:
              name = f"{name}_{pd.Timestamp.now().strftime("%Y-%m-%d_%H-%M-%S")}"
         try:
            wandb.login(key=api_key)
            run = wandb.init(
                 entity=entity,
                 group=group,
                 job_type=job_type,
                 resume=resume,
                 reinit=reinit,
                 
                 project=project_name,   
            )
            try:
                yield run
            finally:
                run.finish()
         except Exception as e:
              logs = Utils.get_loggers(__name__)
              logs.error(f"Couldn't init The weight and bases please check again {e}")
              raise
         
    def check_artifacts_exists(
              artifact_name:Optional[str]=None,
              project_name : Optional[str]=os.environ.get("WDB_PROJECT_NAME"),
              entity_name:Optional[str]=os.environ.get("WDB_ENTITY_NAME")
    ):
         """Helper function to get The artifacts"""
         try:
              Utils.get_artifacts(artifact_name,project_name,entity_name)
              return True
         except wandb.errors.CommError:
              return False
         
   



 
    